#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
    Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
    JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
    Repeat, RepeatDate, RepeatDateTime, RepeatDateList, RepeatDay, RepeatEnumerated, RepeatInteger, \
    RepeatString, SState, State, Style, Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, \
    TimeSlot, Today, UrlCmd, Variable, VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, \
    ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest
import sys


class Test_dunder_rshift(unittest.TestCase):
    def test_node_dunder_rshift(self):
        # will ONLY work if we have starting NodeContainer
        suite = Suite('s') >> Task('t1') >> Task('t2') >> Task('t3') >> Task('t4')
        self.assertEqual(len(list(suite)), 4, "expected 4 children but found " + str(len(list(suite))))

        self.assertEqual(str(suite.t2.get_trigger()), "t1 == complete", "Trigger not as expected: " + str(suite.t2.get_trigger()))
        self.assertEqual(str(suite.t3.get_trigger()), "t2 == complete", "Trigger not as expected: " + str(suite.t3.get_trigger()))
        self.assertEqual(str(suite.t4.get_trigger()), "t3 == complete", "Trigger not as expected: " + str(suite.t4.get_trigger()))

        fam = Family("f1") >> Task('t1') >> Task('t2') >> Task('t3') >> Task('t4')
        self.assertEqual(len(list(fam)), 4, "expected 4 children but found " + str(len(list(fam))))

        self.assertEqual(str(fam.t2.get_trigger()), "t1 == complete", "Trigger not as expected: " + str(fam.t2.get_trigger()))
        self.assertEqual(str(fam.t3.get_trigger()), "t2 == complete", "Trigger not as expected: " + str(fam.t3.get_trigger()))
        self.assertEqual(str(fam.t4.get_trigger()), "t3 == complete", "Trigger not as expected: " + str(fam.t4.get_trigger()))

    def test_node_dunder_rshift_trigger_cat(self):
        suite = Suite('s')
        # will ONLY work if we have starting NodeContainer
        suite >> Task('t1') >> (Task('t2') + Trigger("x == 1")) >> (Task('t3') + Trigger("y==1")) >> Task('t4')
        self.assertEqual(len(list(suite)), 4, "expected 4 children but found " + str(len(list(suite))))

        self.assertEqual(str(suite.t2.get_trigger()), "x == 1 AND t1 == complete", "Trigger not as expected: " + str(suite.t2.get_trigger()))
        self.assertEqual(str(suite.t3.get_trigger()), "y==1 AND t2 == complete", "Trigger not as expected: " + str(suite.t3.get_trigger()))
        self.assertEqual(str(suite.t4.get_trigger()), "t3 == complete", "Trigger not as expected: " + str(suite.t4.get_trigger()))


class Test_dunder_lshift(unittest.TestCase):
    def test_node_dunder_lshift(self):
        # will ONLY work if we have starting NodeContainer
        suite = Suite('s') << Task('t1') << Task('t2') << Task('t3') << Task('t4')
        self.assertEqual(len(list(suite)), 4, "expected 4 children but found " + str(len(list(suite))))

        self.assertEqual(suite.t4.get_trigger(), None, "Trigger not as expected: " + str(suite.t4.get_trigger()))
        self.assertEqual(str(suite.t3.get_trigger()), "t4 == complete", "Trigger not as expected: " + str(suite.t3.get_trigger()))
        self.assertEqual(str(suite.t2.get_trigger()), "t3 == complete", "Trigger not as expected: " + str(suite.t2.get_trigger()))
        self.assertEqual(str(suite.t1.get_trigger()), "t2 == complete", "Trigger not as expected: " + str(suite.t1.get_trigger()))

        fam = Family("f1") << Task('t1') << Task('t2') << Task('t3') << Task('t4')
        self.assertEqual(len(list(fam)), 4, "expected 4 children but found " + str(len(list(fam))))

        self.assertEqual(fam.t4.get_trigger(), None, "Trigger not as expected: " + str(fam.t4.get_trigger()))
        self.assertEqual(str(fam.t3.get_trigger()), "t4 == complete", "Trigger not as expected: " + str(fam.t3.get_trigger()))
        self.assertEqual(str(fam.t2.get_trigger()), "t3 == complete", "Trigger not as expected: " + str(fam.t2.get_trigger()))
        self.assertEqual(str(fam.t1.get_trigger()), "t2 == complete", "Trigger not as expected: " + str(fam.t1.get_trigger()))

    def test_node_dunder_lshift_trigger_cat(self):
        suite = Suite('s')
        # will ONLY work if we have starting NodeContainer
        suite >> Task('t1') << (Task('t2') + Trigger("x == 1")) << (Task('t3') + Trigger("y==1")) << Task('t4')
        self.assertEqual(len(list(suite)), 4, "expected 4 children but found " + str(len(list(suite))))

        self.assertEqual(suite.t4.get_trigger(), None, "Trigger not as expected: " + str(suite.t4.get_trigger()))
        self.assertEqual(str(suite.t3.get_trigger()), "y==1 AND t4 == complete", "Trigger not as expected: " + str(suite.t3.get_trigger()))
        self.assertEqual(str(suite.t2.get_trigger()), "x == 1 AND t3 == complete", "Trigger not as expected: " + str(suite.t2.get_trigger()))
        self.assertEqual(str(suite.t1.get_trigger()), "t2 == complete", "Trigger not as expected: " + str(suite.t1.get_trigger()))


class Test_dunder_add(unittest.TestCase):

    def test_node_dunder_add_variable(self):
        # will ONLY work if we have starting NodeContainer
        suite = Suite("s") + Family("f") + Family("f2") + Task("t3") + Edit(name="value")
        self.assertEqual(len(list(suite.variables)), 1, "expected suite to have 1 variable " + str(len(list(suite.variables))))

        suite = Suite("s") + Family("f") + Family("f2") + (Task("t3") + Edit(name="value"))
        self.assertEqual(len(list(suite.t3.variables)), 1, "expected task t3 to have 1 variable " + str(len(list(suite.t3.variables))))

    def test_defs_dunder_add(self):
        # will ONLY work if we have starting defs
        defs = Defs() + Suite("s") + Suite("s1")
        self.assertEqual(len(defs), 2, "expected 2 suites but found " + str(len(defs)))

        defs += Suite("sx")  # + Suite('x')  wont work,   Suite('sx').__add__(Suite('x') not defined
        self.assertEqual(len(defs), 3, "expected 3 suites but found " + str(len(defs)))

        defs + Suite("a") + Suite("b")
        self.assertEqual(len(defs), 5, "expected 5 suites but found " + str(len(defs)))

    def test_node_dunder_add_meter(self):
        # will ONLY work if we have starting NodeContainer
        suite = Suite("s") + Family("f") + Family("f2") + Task("t3") + Edit(name="value")
        suite.t3 + Event(11, "event") + Meter("meter", 0, 10, 10) + Label("label", "c") + Trigger("1==1")
        self.assertEqual(len(suite), 3, "expected 3 children but found " + str(len(suite)))
        self.assertEqual(len(list(suite.t3.events)), 1, "expected 1 event found " + str(len(list(suite.t3.events))))
        self.assertEqual(len(list(suite.t3.meters)), 1, "expected 1 event found " + str(len(list(suite.t3.meters))))
        self.assertEqual(len(list(suite.t3.labels)), 1, "expected 1 event found " + str(len(list(suite.t3.labels))))

    def test_node_dunder_add(self):
        suite = Suite("s")
        suite += Task("t3")  # + Task("t2") not allowed as Task("t1").__add__("t2") not defined
        suite.t3 += Event(11, "event")  # + Task("t2") not allowed as Task("t1").__add__("t2") not defined
        self.assertEqual(len(suite), 1, "expected 1 children but found " + str(len(suite)))
        self.assertEqual(len(list(suite.t3.events)), 1, "expected 1 event found " + str(len(list(suite.t3.events))))

        suite = Suite('s') + [Task("t{0}".format(i)) for i in range(1, 5)]
        self.assertEqual(len(suite), 4, "expected 4 children but found " + str(len(suite)))

    def test_dunder_add_all(self):
        defs = Defs() + Suite("s1")
        defs.s1 += Clock(1, 1, 2010, False)
        defs.s1 += Autocancel(1, 10, True)
        defs.s1 += Task('t1') + Edit({"a": "y", "b": "bb"}, c="v", d="b") + Edit({"e": 1, "f": "bb"}) + Edit(g="d") + Edit(h=1) + \
                   Event(1) + Event(11, "event") + Meter("meter", 0, 10, 10) + Label("label", "c") + Trigger("1==1") + \
                   Complete("1==1") + Limit("limit", 10) + Limit("limit2", 10) + InLimit("limitName", "/limit", 2) + \
                   Defstatus(DState.complete) + Today(0, 30) + Today("00:59") + Today("00:00 11:30 00:01") + \
                   Time(0, 30) + Time("00:59") + Time("00:00 11:30 00:01") + Day("sunday") + Day(Days.monday) + \
                   Date(1, 1, 0) + Date(28, 2, 1960) + Autocancel(3)

        t1 = defs.find_abs_node("/s1/t1")
        self.assertTrue(t1 is not None, "Can't find t1")
        self.assertEqual(len(defs.s1), 1, "Expected 1 nodes but found " + str(len(defs.s1)))
        self.assertEqual(len(list(t1.variables)), 8, "expected 8 variables but found " + str(len(list(t1.variables))))
        self.assertEqual(len(list(t1.limits)), 2, "expected 2 limits")
        self.assertEqual(len(list(t1.inlimits)), 1, "expected 1 inlimits")
        self.assertEqual(len(list(t1.events)), 2, "expected 2 events")
        self.assertEqual(len(list(t1.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(t1.labels)), 1, "expected 1 label")
        self.assertEqual(len(list(t1.times)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.todays)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.days)), 2, "expected 2 days")
        self.assertEqual(len(list(t1.dates)), 2, "expected 2 dates")
        self.assertTrue(t1.get_trigger(), "Can't find t1 trigger")
        self.assertTrue(t1.get_complete(), "Can't find t1 complete")
        self.assertTrue(t1.get_autocancel(), "Can't find t1 autocancel")
        # print(defs)


class Test_crash(unittest.TestCase):
    def test_trigger_node_list(self):
        suite = Suite("s")
        fam = suite.add_family("f")
        task = fam.add_task("t")
        self.assertEqual(task.get_abs_node_path(), "/s/f/t", "Path not as expected " + task.get_abs_node_path())

        suite = Suite("s")
        fam = Family("f")
        task = Task("t")
        suite.add_family(fam)
        task = fam.add_task(task)
        self.assertEqual(task.get_abs_node_path(), "/s/f/t", "Path not as expected " + task.get_abs_node_path())

        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        self.assertEqual(task.get_abs_node_path(), "/s/f/t", "Path not as expected " + task.get_abs_node_path())

        # The following will crash or cause memory error. Since the Defs() is transitory/temp()
        # Unlike the previous example where we keep the defs in a variable.
        # task = Defs().add_suite("s").add_family("f").add_task("t")
        # self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )

        # Like wise this will crash since the Suite() is transitory reference counted, and goes out of scope.


#         task = Suite("s").add_family("f").add_task("t")
#         self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )

class TestListComprehension(unittest.TestCase):
    def test_suite_list(self):
        defs = Defs()
        defs += [Suite("s{0}".format(i)) for i in range(1, 6)]
        self.assertEqual(len(defs), 5, " expected 5 suites but found " + str(len(defs)))

        defs.add([Suite("s{0}".format(i)) for i in range(6, 11)])
        self.assertEqual(len(defs), 10, " expected 10 suites but found " + str(len(defs)))

        defs = Defs([Suite("s{0}".format(i)) for i in range(1, 6)])
        self.assertEqual(len(defs), 5, " expected 5 suites but found " + str(len(defs)))

    def test_family_list(self):
        defs = Defs()
        defs += [Suite("suite").add([Family("f{0}".format(i)) for i in range(1, 6)])]
        self.assertEqual(len(defs.suite), 5, " expected 5 familes but found " + str(len(defs.suite)))

        defs = Defs([Suite("xx", [Family("f{0}".format(i)) for i in range(1, 6)])])
        self.assertEqual(len(defs.xx), 5, " expected 5 familes but found " + str(len(defs.xx)))

    def test_task_list(self):
        defs = Defs()
        defs += [Suite("suite").add(Family("f").add([Task("t{0}".format(i)) for i in range(1, 6)]))]
        self.assertEqual(len(defs.suite.f), 5, " expected 5 task but found " + str(len(defs.suite.f)))

        defs = Defs([Suite("suite", Family("f", [Task("t{0}".format(i)) for i in range(1, 6)]))])
        self.assertEqual(len(defs.suite.f), 5, " expected 5 task but found " + str(len(defs.suite.f)))

    def test_task_list2(self):
        defs = Defs()
        defs += [Suite("suite").add(Task("x"),
                                    Family("f").add([Task("t{0}".format(i)) for i in range(1, 6)]),
                                    Task("y"),
                                    [Family("f{0}".format(i)) for i in range(1, 6)],
                                    Edit(a="b"),
                                    [Task("t{0}".format(i)) for i in range(1, 6)],
                                    )
                 ]
        self.assertEqual(len(defs.suite), 13, " expected 13 nodes but found " + str(len(defs.suite)))
        self.assertEqual(len(defs.suite.f), 5, " expected 5 nodes but found " + str(len(defs.suite.f)))
        self.assertEqual(len(list(defs.suite.variables)), 1, " expected 1 variable " + str(len(list(defs.suite.variables))))

    def test_5Suite_with_5families_with_5tasks(self):
        defs = Defs().add(
            [Suite("s{0}".format(i)).add(
                [Family("f{0}".format(i)).add(
                    [Task("t{0}".format(i))
                     for i in range(1, 6)])
                    for i in range(1, 6)])
                for i in range(1, 6)])
        # print defs
        self.assertEqual(len(defs), 5, " expected 5 suites but found " + str(len(defs)))
        for suite in defs:
            self.assertEqual(len(suite), 5, " expected 5 familes but found " + str(len(suite)))
            for fam in suite:
                self.assertEqual(len(fam), 5, " expected 5 tasks but found " + str(len(fam)))

    def test__5Suite_with_5families_with_5tasks(self):
        defs = Defs()
        defs += [Suite("s{0}".format(i)).add(
            [Family("f{0}".format(i)).add(
                [Task("t{0}".format(i)) for i in range(1, 6)])
                for i in range(1, 6)])
            for i in range(1, 6)
        ]
        # print defs
        self.assertEqual(len(defs), 5, " expected 5 suites but found " + str(len(defs)))
        for suite in defs:
            self.assertEqual(len(suite), 5, " expected 5 familes but found " + str(len(suite)))
            for fam in suite:
                self.assertEqual(len(fam), 5, " expected 5 tasks but found " + str(len(fam)))


class TestTrigger(unittest.TestCase):
    def test_trigger(self):
        t = Trigger("a == complete")
        self.assertEqual(str(t), "a == complete", "Trigger not as expected:" + str(t))

    def test_trigger_string_list(self):
        t = Trigger(["a"])
        self.assertEqual(str(t), "a == complete", "Trigger not as expected: " + str(t))

        t = Trigger(["a == 1"])
        self.assertEqual(str(t), "a == 1", "Trigger not as expected: " + str(t))

        t = Trigger(["a", "b", "c"])
        self.assertEqual(str(t), "a == complete AND b == complete AND c == complete", "Trigger not as expected: " + str(t))

    def test_trigger_node_list(self):
        t = Trigger([Task("a")])
        self.assertEqual(str(t), "a == complete", "Trigger not as expected: " + str(t))

        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        self.assertEqual(task.get_abs_node_path(), "/s/f/t", "Path not as expected " + task.get_abs_node_path())
        t = Trigger([task])
        self.assertEqual(str(t), "/s/f/t == complete", "Trigger not as expected: " + str(t))

    def test_trigger_string_and_node_list(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        t = Trigger(["a", "b", task])
        self.assertEqual(str(t), "a == complete AND b == complete AND /s/f/t == complete", "Trigger not as expected: " + str(t))
        task += t
        self.assertEqual(str(task.get_trigger()), "a == complete AND b == complete AND /s/f/t == complete", "Trigger not as expected: " + str(task.get_trigger()))

    def test_add(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        task.add(Trigger("1==1"))
        expr = task.get_trigger()
        self.assertEqual(str(expr), "1==1", "Trigger not as expected: " + expr.get_expression())

        task.add(Trigger("2==1"))
        self.assertEqual(str(expr), "1==1 AND 2==1", "Trigger not as expected: " + str(expr))

    def test_add_composition(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        task.add(Trigger("1==1"))
        task.add(Trigger("1==2", False))
        task.add(Trigger("x==2"))
        expr = task.get_trigger()
        self.assertEqual(str(expr), "1==1 OR 1==2 AND x==2", "Trigger not as expected: " + expr.get_expression())


class TestComplete(unittest.TestCase):
    def test_trigger(self):
        t = Complete("a == complete")
        self.assertEqual(str(t), "a == complete", "Complete not as expected:" + str(t))

    def test_Complete_string_list(self):
        t = Complete(["a"])
        self.assertEqual(str(t), "a == complete", "Complete not as expected: " + str(t))

        t = Complete(["a", "b", "c"])
        self.assertEqual(str(t), "a == complete AND b == complete AND c == complete", "Complete not as expected: " + str(t))

    def test_Complete_node_list(self):
        t = Complete([Task("a")])
        self.assertEqual(str(t), "a == complete", "Complete not as expected: " + str(t))

        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        self.assertEqual(task.get_abs_node_path(), "/s/f/t", "Path not as expected " + task.get_abs_node_path())
        t = Complete([task])
        self.assertEqual(str(t), "/s/f/t == complete", "Complete not as expected: " + str(t))

    def test_Complete_string_and_node_list(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        t = Complete(["a", "b", task])
        self.assertEqual(str(t), "a == complete AND b == complete AND /s/f/t == complete", "Complete not as expected: " + str(t))

    def test_add(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        task.add(Complete("1==1"))
        expr = task.get_complete()
        self.assertEqual(str(expr), "1==1", "Complete not as expected: " + expr.get_expression())

        task.add(Complete("2==1"))
        self.assertEqual(str(expr), "1==1 AND 2==1", "Complete not as expected: " + str(expr))

    def test_add_composition(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        task.add(Complete("1==1"))
        task.add(Complete("1==2", False))
        task.add(Complete("x==2"))
        expr = task.get_complete()
        self.assertEqual(str(expr), "1==1 OR 1==2 AND x==2", "Trigger not as expected: " + expr.get_expression())


class TestDefsAdd(unittest.TestCase):
    def test_add_suite1(self):
        defs = Defs().add(Suite("a"))
        self.assertEqual(len(defs), 1, "Expected 1 suite to be added")

    def test_add_suite3(self):
        defs = Defs().add(Suite("a"), Suite("b"), Suite("c").add(Edit(a="b")))
        self.assertEqual(len(defs), 3, "Expected 3 suite to be added")

    def test_add_variables(self):
        defs = Defs().add(Edit(a="b", b='c', d='e'))
        self.assertEqual(len(list(defs.user_variables)), 3, "Expected 3 variables to be added")

    def test_add_variables2(self):
        defs = Defs(Edit(a="b", b='c', d='e'))
        self.assertEqual(len(list(defs.user_variables)), 3, "Expected 3 variables to be added")


class TestSuiteAdd(unittest.TestCase):
    def test_add_suite1(self):
        s = Suite("a").add(Family("a").add(Edit(a="b")))
        self.assertEqual(len(s), 1, "Expected 1 child to be added")


class TestFamilyAdd(unittest.TestCase):
    def test_add_task(self):
        s = Family("a").add(Task("a").add(Edit(a="b")))
        self.assertEqual(len(s), 1, "Expected 1 child to be added")


class TestDefstatus(unittest.TestCase):
    def test_illegal_defstatus(self):
        if sys.version_info[0] == 2 and sys.version_info[1] >= 7:
            with self.assertRaises(RuntimeError):
                Defstatus("fred")


class TestEdit(unittest.TestCase):
    def test_add_variable(self):
        defs = Defs().add(Suite("s").add(Edit(a=1, b=1), Edit(c="c")))
        self.assertEqual(len(list(defs.s.variables)), 3, "expected 3 variables")

    def test_edit(self):
        t = Task("t1").add(
            Edit({"a": "y", "b": "bb"}, c="v", d="b"),
            Edit({"e": "1", "f": "bb"}),
            Edit(g="d"),
            Edit(h="1"))
        self.assertEqual(len(list(t.variables)), 8, "expected 8 variables but found " + str(len(list(t.variables))))


class TestAddAll(unittest.TestCase):
    def test_add(self):
        defs = Defs().add(
            Suite("s1").add(
                Clock(1, 1, 2010, False),
                Autocancel(1, 10, True),
                Task("t1").add(
                    Edit({"a": "y", "b": "bb"}, c="v", d="b"),
                    Edit({"e": 1, "f": "bb"}),
                    Edit(g="d"),
                    Edit(h=1),
                    Event(1),
                    Event(11, "event"),
                    Meter("meter", 0, 10, 10),
                    Label("label", "c"),
                    Trigger("1==1"),
                    Complete("1==1"),
                    Limit("limit", 10), Limit("limit2", 10),
                    InLimit("limitName", "/limit", 2),
                    Defstatus(DState.complete),
                    Today(0, 30), Today("00:59"), Today("00:00 11:30 00:01"),
                    Time(0, 30), Time("00:59"), Time("00:00 11:30 00:01"),
                    Day("sunday"), Day(Days.monday),
                    Date(1, 1, 0), Date(28, 2, 1960),
                    Autocancel(3)
                ),
                [Family("f{0}".format(i)) for i in range(1, 6)]
            )
        )
        t1 = defs.find_abs_node("/s1/t1")
        self.assertTrue(t1 is not None, "Can't find t1")
        self.assertEqual(len(defs.s1), 6, "Expected 6 nodes but found " + str(len(defs.s1)))
        self.assertEqual(len(list(t1.variables)), 8, "expected 8 variables")
        self.assertEqual(len(list(t1.limits)), 2, "expected 2 limits")
        self.assertEqual(len(list(t1.inlimits)), 1, "expected 1 inlimits")
        self.assertEqual(len(list(t1.events)), 2, "expected 2 events")
        self.assertEqual(len(list(t1.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(t1.labels)), 1, "expected 1 label")
        self.assertEqual(len(list(t1.times)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.todays)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.days)), 2, "expected 2 days")
        self.assertEqual(len(list(t1.dates)), 2, "expected 2 dates")
        self.assertTrue(t1.get_trigger(), "Can't find t1 trigger")
        self.assertTrue(t1.get_complete(), "Can't find t1 complete")
        self.assertTrue(t1.get_autocancel(), "Can't find t1 autocancel")
        # print(defs)

    def test_raw_constructor(self):
        defs = Defs().add(
            Suite("s1",
                  Clock(1, 1, 2010, False),
                  Autocancel(1, 10, True),
                  Task("t1",
                       Edit({"a": "y", "b": "bb"}, c="v", d="b"),
                       Edit({"e": 1, "f": "bb"}),
                       Edit(g="d"),
                       Edit(h=1),
                       Event(1),
                       Event(11, "event"),
                       Meter("meter", 0, 10, 10),
                       Label("label", "c"),
                       Trigger("1==1"),
                       Complete("1==1"),
                       Limit("limit", 10), Limit("limit2", 10),
                       InLimit("limitName", "/limit", 2),
                       Defstatus(DState.complete),
                       Today(0, 30), Today("00:59"), Today("00:00 11:30 00:01"),
                       Time(0, 30), Time("00:59"), Time("00:00 11:30 00:01"),
                       Day("sunday"), Day(Days.monday),
                       Date(1, 1, 0), Date(28, 2, 1960),
                       Autocancel(3)
                       ),
                  [Family("f{0}".format(i)) for i in range(1, 6)]
                  )
        )
        print(defs)
        t1 = defs.find_abs_node("/s1/t1")
        self.assertTrue(t1 is not None, "Can't find t1")
        self.assertEqual(len(defs.s1), 6, "Expected 6 nodes but found " + str(len(defs.s1)))
        self.assertEqual(len(list(t1.variables)), 8, "expected 8 variables")
        self.assertEqual(len(list(t1.limits)), 2, "expected 2 limits")
        self.assertEqual(len(list(t1.inlimits)), 1, "expected 1 inlimits")
        self.assertEqual(len(list(t1.events)), 2, "expected 2 events")
        self.assertEqual(len(list(t1.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(t1.labels)), 1, "expected 1 label")
        self.assertEqual(len(list(t1.times)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.todays)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.days)), 2, "expected 2 days")
        self.assertEqual(len(list(t1.dates)), 2, "expected 2 dates")
        self.assertTrue(t1.get_trigger(), "Can't find t1 trigger")
        self.assertTrue(t1.get_complete(), "Can't find t1 complete")
        self.assertTrue(t1.get_autocancel(), "Can't find t1 autocancel")


class TestIAdd(unittest.TestCase):
    def test_iadd_with_getattr(self):
        defs = Defs();
        s1 = defs.add_suite("s1")
        s1 += [Task("a"), Task("b")]
        defs.s1.a += [Edit({"x1": "y", "aa1": "bb"}, a="v", b="b", ),
                      Edit({"x": "y", "aa": "bb"}),
                      Edit(d="d")
                      ]
        defs.s1.b += [Event(1),
                      Event(11, "event"),
                      Meter("meter", 0, 10, 10),
                      Label("label", "c"),
                      ]
        self.assertEqual(len(list(defs.s1.a.variables)), 7, "expected 7 variables")
        self.assertEqual(len(list(defs.s1.b.events)), 2, "expected 2 events")
        self.assertEqual(len(list(defs.s1.b.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(defs.s1.b.labels)), 1, "expected 1 label")

    def test_iadd(self):
        defs = Defs();
        defs += [Suite("s2"), Edit({"x1": "y", "aa1": "bb"}, a="v", b="b", )]
        s1 = defs.add_suite("s1")
        s1 += [Clock(1, 1, 2010, False), Autocancel(1, 10, True)]  # + returns the same node back
        t1 = s1.add_task("t1")
        t1 += [Edit({"x1": "y", "aa1": "bb"}, a="v", b="b", ),
               Edit({"x": "y", "aa": "bb"}),
               Edit(d="d"),
               Event(1),
               Event(11, "event"),
               Meter("meter", 0, 10, 10),
               Label("label", "c"),
               Trigger("1==1"),
               Complete("1==1"),
               Limit("limit", 10), Limit("limit2", 10),
               InLimit("limitName", "/limit", 2),
               Defstatus(DState.complete),
               Today(0, 30), Today("00:59"), Today("00:00 11:30 00:01"),
               Time(0, 30), Time("00:59"), Time("00:00 11:30 00:01"),
               Day("sunday"), Day(Days.monday),
               Date(1, 1, 0), Date(28, 2, 1960),
               Autocancel(3)
               ]
        self.assertTrue(t1 is not None, "Can't find t1")
        self.assertEqual(len(list(defs.suites)), 2, "expected 7 suites")
        self.assertEqual(len(list(t1.variables)), 7, "expected 7 variables")
        self.assertEqual(len(list(t1.limits)), 2, "expected 2 limits")
        self.assertEqual(len(list(t1.inlimits)), 1, "expected 1 inlimits")
        self.assertEqual(len(list(t1.events)), 2, "expected 2 events")
        self.assertEqual(len(list(t1.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(t1.labels)), 1, "expected 1 label")
        self.assertEqual(len(list(t1.times)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.todays)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.days)), 2, "expected 2 days")
        self.assertEqual(len(list(t1.dates)), 2, "expected 2 dates")
        self.assertTrue(t1.get_trigger(), "Can't find t1 trigger")
        self.assertTrue(t1.get_complete(), "Can't find t1 complete")
        self.assertTrue(t1.get_autocancel(), "Can't find t1 autocancel")
        # print(defs)


class TestComparison(unittest.TestCase):
    def setUp(self):
        self.cron = Cron()
        self.cron.set_time_series(TimeSeries(TimeSlot(23, 0), True))  # True means relative to suite start
        self.late = Late()
        self.late.submitted(TimeSlot(20, 10))
        self.late.active(TimeSlot(20, 10))
        self.late.complete(TimeSlot(20, 10), True)
        self.late2 = Late()
        self.late2.submitted(20, 10)
        self.late2.active(20, 10)
        self.late2.complete(20, 10, True)

        self.defs1 = Defs()
        self.defs1.add_suite("s1").add_task("t1").add_variable("a", "v").add_event(1).add_event(11, "event").add_meter("meter", 0, 10, 10).add_label("label", "c")
        self.defs1.add_suite("s2").add_family("f1").add_task("t1").add_trigger("1==1").add_complete("1==1")
        self.defs1.add_suite("s3").add_family("f1").add_family("f2").add_task("t1").add_variable("var", "v")
        self.defs1.add_suite("edit").add_variable("a", "a").add_variable("b", "b")
        self.defs1.add_suite("limit").add_limit("limit", 10).add_limit("limit2", 10)
        self.defs1.add_suite("inlimit").add_inlimit("limitName", "/limit", 2)
        self.defs1.add_suite("RepeatInteger").add_repeat(RepeatInteger("integer", 0, 100, 2))
        self.defs1.add_suite("RepeatEnumerated").add_repeat(RepeatEnumerated("enum", ["red", "green", "blue"]))
        self.defs1.add_suite("RepeatDate").add_repeat(RepeatDate("ymd", 20100111, 20100115, 2))
        self.defs1.add_suite("RepeatDateTime").add_repeat(RepeatDateTime("ymd", "20100111T000000", "20100115T000000", "48:00:00"))
        self.defs1.add_suite("RepeatDateList").add_repeat(RepeatDateList("ymd", [20100111, 20100115]))
        self.defs1.add_suite("RepeatString").add_repeat(RepeatString("string", ["a", "b", "c"]))
        self.defs1.add_suite("RepeatDay").add_repeat(RepeatDay(1))
        self.defs1.add_suite("defstatus").add_defstatus(DState.active)
        self.defs1.add_suite("today").add_task("today").add_today("00:30").add_today(0, 59).add_today("00:00 11:30 00:01")
        self.defs1.add_suite("time").add_task("time").add_time("00:30").add_time(0, 59).add_time("00:00 11:30 00:01")
        self.defs1.add_suite("day").add_task("day").add_day("sunday").add_day(Days.monday)
        self.defs1.add_suite("date").add_task("date").add_date(1, 1, 0).add_date(28, 2, 1960)
        self.defs1.add_suite("cron").add_task("cron").add_cron(self.cron)
        self.defs1.add_suite("late").add_family("late").add_late(self.late).add_task("late").add_late(self.late2)
        self.defs1.add_suite("clock").add_clock(Clock(1, 1, 2010, False))
        self.defs1.add_suite("autocancel").add_autocancel(3)

        zombie_suite = self.defs1.add_suite("zombie")
        self.zombie_life_time_in_server = 800
        self.child_list = [ChildCmdType.init, ChildCmdType.event, ChildCmdType.meter, ChildCmdType.label, ChildCmdType.wait, ChildCmdType.abort, ChildCmdType.complete]
        zombie_type_list = [ZombieType.ecf, ZombieType.user, ZombieType.path]
        for zombie_type in zombie_type_list:
            zombie_attr = ZombieAttr(zombie_type, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server)
            zombie_suite.add_zombie(zombie_attr)

    def test_compare_with_add(self):
        defs = Defs().add(
            Suite("s1").add(
                Task("t1").add(
                    Edit(a="v"),
                    Event(1),
                    Event(11, "event"),
                    Meter("meter", 0, 10, 10),
                    Label("label", "c")
                )
            ),
            Suite("s2").add(
                Family("f1").add(
                    Task("t1").add(
                        Trigger("1==1"),
                        Complete("1==1")
                    )
                )
            ),
            Suite("s3").add(
                Family("f1").add(
                    Family("f2").add(
                        Task("t1").add(
                            Edit(var="v")
                        )
                    )
                )
            ),
            Suite("edit").add(Edit({"a": "a"}), Edit({"b": "b"})),  # add separate, otherwise with dict order is undefined
            Suite("limit").add(Limit("limit", 10), Limit("limit2", 10)),
            Suite("inlimit").add(InLimit("limitName", "/limit", 2)),
            Suite("RepeatInteger").add(RepeatInteger("integer", 0, 100, 2)),
            Suite("RepeatEnumerated").add(RepeatEnumerated("enum", ["red", "green", "blue"])),
            Suite("RepeatDate").add(RepeatDate("ymd", 20100111, 20100115, 2)),
            Suite("RepeatDateTime").add(RepeatDateTime("ymd", "20100111T000000", "20100115T000000", "48:00:00")),
            Suite("RepeatDateList").add(RepeatDateList("ymd", [20100111, 20100115])),
            Suite("RepeatString").add(RepeatString("string", ["a", "b", "c"])),
            Suite("RepeatDay").add(RepeatDay(1)),
            Suite("defstatus").add(Defstatus(DState.active)),
            Suite("today").add(
                Task("today").add(
                    Today(0, 30), Today("00:59"), Today("00:00 11:30 00:01")
                )
            ),
            Suite("time").add(
                Task("time").add(
                    Time(0, 30), Time("00:59"), Time("00:00 11:30 00:01")
                )
            ),
            Suite("day").add(
                Task("day").add(
                    Day("sunday"), Day(Days.monday)
                )
            ),
            Suite("date").add(
                Task("date").add(
                    Date(1, 1, 0), Date(28, 2, 1960)
                )
            ),
            Suite("cron").add(
                Task("cron").add(
                    self.cron
                )
            ),
            Suite("late").add(
                Family("late").add(
                    self.late,
                    Task("late").add(
                        self.late2
                    )
                )
            ),
            Suite("clock").add(Clock(1, 1, 2010, False)),
            Suite("autocancel").add(Autocancel(3)),
            Suite("zombie").add(
                ZombieAttr(ZombieType.ecf, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server),
                ZombieAttr(ZombieType.user, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server),
                ZombieAttr(ZombieType.path, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server)
            )
        )

        self.assertEqual(self.defs1, defs, "defs not equal\n" + str(self.defs1) + "\n\n" + str(defs))

    def test_compare_with_raw_constructor(self):
        defs = Defs()
        defs += Suite('s1', Task('t1', Event(1), Event(11, 'event'), Meter("meter", 0, 10, 10), Label("label", "c"), a='v'))
        defs += Suite('s2', Family('f1', Task('t1', Trigger("1==1"), Complete("1==1"))))
        defs += Suite('s3', Family('f1', Family('f2', Task('t1', var='v'))))
        defs += Suite('edit', Edit(a='a'), Edit(b='b'))
        defs += Suite('limit', Limit("limit", 10), Limit("limit2", 10))
        defs += Suite('inlimit', InLimit("limitName", "/limit", 2))
        defs += Suite('RepeatInteger', RepeatInteger("integer", 0, 100, 2))
        defs += Suite('RepeatEnumerated', RepeatEnumerated("enum", ["red", "green", "blue"]))
        defs += Suite('RepeatDate', RepeatDate("ymd", 20100111, 20100115, 2))
        defs += Suite('RepeatDateTime', RepeatDateTime("ymd", "20100111T000000", "20100115T000000", "48:00:00"))
        defs += Suite('RepeatDateList', RepeatDateList("ymd", [20100111, 20100115]))
        defs += Suite('RepeatString', RepeatString("string", ["a", "b", "c"]))
        defs += Suite('RepeatDay', RepeatDay(1))
        defs += Suite('defstatus', Defstatus('active'))
        defs += Suite('today', Task('today', Today("00:30"), Today(0, 59), Today("00:00 11:30 00:01")))
        defs += Suite('time', Task('time', Time("00:30"), Time(0, 59), Time("00:00 11:30 00:01")))
        defs += Suite('day', Task('day', Day("sunday"), Day(Days.monday)))
        defs += Suite('date', Task('date', Date(1, 1, 0), Date(28, 2, 1960)))
        defs += Suite('cron', Task('cron', self.cron))
        defs += Suite('late', Family('late', self.late, Task('late', self.late2)))
        defs += Suite('clock', Clock(1, 1, 2010, False))
        defs += Suite('autocancel', Autocancel(3))

        defs += Suite("zombie", ZombieAttr(ZombieType.ecf, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server),
                      ZombieAttr(ZombieType.user, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server),
                      ZombieAttr(ZombieType.path, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server))

        self.assertEqual(self.defs1, defs, "defs not equal\n" + str(self.defs1) + "\n\n" + str(defs))


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

# This test ensures the creation of Defs objects and their components works as expected.

import os
import ecflow

print(ecflow.__doc__)
print(ecflow.Defs.__doc__)

from ecflow import Suite, Family, Task, Defs, Clock, DState, PartExpression, Variable, Limit, InLimit, \
    Date, Day, Event, Meter, Label, Autocancel, Days, TimeSlot, TimeSeries, Style, State, \
    RepeatString, RepeatDate, RepeatDateTime, RepeatInteger, RepeatDay, RepeatEnumerated, \
    Verify, PrintStyle, Time, Today, Late, Cron, Client, debug_build, Ecf
import ecflow_test_util as Test

if __name__ == "__main__":

    Test.print_test_start(os.path.basename(__file__))

    suite = Suite("s1")
    assert (isinstance(suite, ecflow.Suite)), "Expected suite"
    assert (not isinstance(suite, ecflow.Family)), "Expected suite"
    assert (not isinstance(suite, ecflow.Task)), "Expected suite"

    family = Family("f1")
    assert (isinstance(family, ecflow.Family)), "Expected Family"
    assert (not isinstance(family, ecflow.Suite)), "Expected Family"
    assert (not isinstance(family, ecflow.Task)), "Expected Family"
    suite.add_family(family)

    task = Task("f1")
    assert (isinstance(task, ecflow.Task)), "Expected Task"
    assert (not isinstance(task, ecflow.Suite)), "Expected Task"
    assert (not isinstance(task, ecflow.Family)), "Expected Task"
    family.add_task(task)

    defs = Defs()
    defs2 = Defs()
    assert defs == defs2, "Empty defs should be equal"

    assert defs.get_state() == ecflow.State.unknown, "Expected default state to be unknown"
    defs.add_suite(suite);  # add existing suite to defs

    suite = defs.add_suite("s2");
    suite.add_variable(Variable("ECF_HOME", "/tmp/"))
    suite.add_variable("ECF_URL_CMD", "${BROWSER:=firefox} -new-tab %ECF_URL_BASE%/%ECF_URL%")
    suite.add_variable("ECF_URL_BASE", "http://www.ecmwf.int")
    suite.add_variable("ECF_URL", "publications/manuals/sms")
    suite.add_limit(Limit("limitName", 10))
    suite.add_limit("limitName_2", 10)
    assert suite.begun() == False, "Suite should not have begun"

    clock = Clock(1, 1, 2010, False);
    clock.set_gain(1, 10, True);
    suite.add_clock(clock);

    assert defs != defs2, "Defs should no longer compare as they are different"

    defs.add_extern("/path/to/task");
    defs.add_extern("/fred/bill:event_name");

    family = suite.add_family("f1")
    family.add_defstatus(DState.active);

    task = family.add_task("t1")
    task.add_trigger("t2 == active")
    task.add_complete("t2 == complete")
    task.add_autocancel(Autocancel(3));

    task2 = family.add_task("t2")
    task2.add_defstatus(DState.complete);
    task2.add_part_trigger(PartExpression("t1 == complete"))
    task2.add_part_trigger(PartExpression("t1 == active", True))  # for long and/or expressions, subsequent expr must be and/or
    task2.add_part_complete(PartExpression("t1 == complete"))
    task2.add_part_complete(PartExpression("t1 == active", False))  # for long and/or expressions, subsequent expr must be and/or
    task2.add_label(Label("labelname", "label1"))
    task2.add_label("labelname2", "label1")
    task2.add_inlimit(InLimit("limitName", "/s1/f1", 2))
    task2.add_inlimit(InLimit("limitName1", "/s1/f1", 2, True))
    task2.add_inlimit("limitName2")
    task2.add_inlimit("limitName3", "path")
    task2.add_inlimit("limitName4", "path", 1)
    task2.add_inlimit("limitName5", "path", 1, True)
    task2.add_event(10)
    task2.add_event(Event(10, "Eventname"))
    task2.add_meter(Meter("metername", 0, 100, 50))
    task2.add_meter("metername2", 0, 100, 50)
    task2.add_date(Date(1, 1, 2010))
    task2.add_date(1, 2, 2010)
    task2.add_day(Day(Days.sunday))
    task2.add_day(Days.monday)
    task2.add_autocancel(Autocancel(TimeSlot(20, 10), True))

    task2.add_today(0, 10)
    task2.add_today(Today(0, 59, True))
    task2.add_today(Today(TimeSlot(20, 10)))
    task2.add_today(Today(TimeSlot(20, 20), False))
    start = TimeSlot(0, 0)
    finish = TimeSlot(23, 0)
    incr = TimeSlot(0, 30)
    ts = TimeSeries(start, finish, incr, True);
    task2.add_today(Today(ts))

    task2.add_time(20, 10)
    task2.add_time(Time(20, 12, True))
    task2.add_time(Time(TimeSlot(20, 12)))
    task2.add_time(Time(TimeSlot(20, 20), True))
    task2.add_time(Time(ts))

    late = Late()
    late.submitted(20, 10)
    late.active(TimeSlot(20, 10))
    late.complete(TimeSlot(20, 10), True)
    task2.add_late(late)

    cron = Cron()
    cron.set_week_days([0, 1, 2, 3, 4, 5, 6])
    cron.set_days_of_month([1, 2, 3, 4, 5, 6])
    cron.set_months([1, 2, 3, 4, 5, 6])
    cron.set_time_series(ts)
    task2.add_cron(cron);

    task3 = family.add_task("t3")
    task3.add_repeat(RepeatDate("testDate", 20100111, 20100115, 2))
    task3.add_defstatus(DState.queued);
    task3.add_verify(Verify(State.complete, 1))
    task3.add_autocancel(Autocancel(20, 10, False))
    task3.add_label(Label("label_name", "label_value"))

    task3_1 = family.add_task("t3_1")
    task3_1.add_repeat(RepeatDate("testDate", 20100111, 20100115))

    task3_2 = family.add_task("t3_2")
    task3_2.add_repeat(RepeatDateTime("testDateTime", "20100111T000000", "20100115T000000"))

    task4 = family.add_task("t4")
    task4.add_repeat(RepeatInteger("testInteger", 0, 100, 2))
    task4.add_defstatus(DState.aborted);

    task4_1 = family.add_task("t4_1")
    task4_1.add_repeat(RepeatInteger("testInteger", 0, 100))

    task5 = family.add_task("t5")
    task5.add_repeat(RepeatEnumerated("testInteger", ["red", "green", "blue"]))
    task5.add_defstatus(DState.submitted);

    task6 = family.add_task("t6")
    task6.add_repeat(RepeatString("test_string", ["a", "b", "c"]))
    task6.add_defstatus(DState.suspended);

    task7 = family.add_task("t7")
    task7.add_repeat(RepeatDay(2))
    task7.add_defstatus(DState.active);

    task8 = family.add_task("t8")
    task8.add_repeat(RepeatDay())
    print("task8.get_try_no()    :string:", task8.get_try_no())
    print("task8.get_int_try_no():int   :", task8.get_int_try_no())

    the_defs = task8.get_defs()

    # Test find      
    assert defs.find_suite("s1") is not None, "Expected to find suite of name s1"
    assert defs.find_suite("sffff1") is None, "Should not be able to find suite of name sfffff1"

    # save the defs file as a check point file and restore it again
    checkpt_file = "py_u_TestDefs.check"
    defs_file = "py_u_TestDefs.def"
    if debug_build():
        checkpt_file = "py_u_TestDefs_debug.check"
        defs_file = "py_u_TestDefs_debug.def"
    defs.save_as_checkpt(checkpt_file);

    restored_checkpt_defs = Defs()
    restored_checkpt_defs.restore_from_checkpt(checkpt_file)

    print("Save and restore as Defs ******************************************************")
    defs.save_as_defs(defs_file)  # default is to save as DEFS
    restored_from_defs = Defs(defs_file)  # restore the defs

    defs.save_as_defs(defs_file, Style.DEFS)
    restored_from_defs = Defs(defs_file)  # restore the defs

    print("Save and restore using STATE ******************************************************")
    defs.save_as_defs(defs_file, Style.STATE)
    restored_from_defs = Defs(defs_file)  # restore the defs

    print("Save and restore using MIGRATE ******************************************************")
    defs.save_as_defs(defs_file, Style.MIGRATE)
    restored_from_defs = Defs(defs_file)  # restore the defs
    assert restored_checkpt_defs == restored_from_defs, "File should be the same"

    print("Print in DEFS style ******************************************************")
    PrintStyle.set_style(Style.DEFS)
    the_string = str(restored_from_defs)
    assert the_string.find("defs_state") == -1, "Print in DEFS style failed"
    print(the_string)

    print("Print in STATE style *****************************************************")
    PrintStyle.set_style(Style.STATE)
    the_string = str(restored_from_defs)
    assert the_string.find("defs_state STATE") != -1, "Print in STATE style failed"
    print(the_string)

    print("Print in MIGRATE style *****************************************************")
    PrintStyle.set_style(Style.MIGRATE)
    the_string = str(restored_from_defs)
    assert the_string.find("defs_state MIGRATE") != -1, "Print in MIGRATE style failed"
    print(the_string)

    # Comment this out if you want to see what the file looked like
    os.remove(checkpt_file)
    os.remove(defs_file)

    # Test job creation failure
    msg = defs.check_job_creation()
    assert len(msg) != 0, "Expected job creation to fail"

    expected_exeption = False;
    try:
        defs.check_job_creation(throw_on_error=True)
    except RuntimeError as e:
        expected_exeption = True
    assert expected_exeption, "expected exception"

    print("All tests pass")



#!/usr/bin/env python3

import pathlib
from ecflow import Defs, Suite, Task, Edit

if __name__ == '__main__':

    base = pathlib.Path.home() / "course"

    print("[1] Creating suite definition")

    defs = Defs(Suite("test", Edit(ECF_HOME=str(base)), Task("t1"), Task("t2")))
    print(defs)

    print("[2] Checking job creation: .ecf -> .job0")
    print(defs.check_job_creation())

    print("[3] Saving definition to file 'test.def'")
    defs.save_as_defs(str(base / "test.def"))



#!/usr/bin/env python3

import pathlib
from ecflow import Client

if __name__ == '__main__':

    base = pathlib.Path.home() / "course"

    try:
        ci = Client()

        print("[1] Delete all suites from the server")
        ci.delete_all()  # clear out the server

        print("[2] Load suite definition in 'test.def' into the server")
        ci.load(str(base / "test.def"))
        # line above reads suite definition from disk and loads into the server

        print("[3] Begin the 'test' suite")
        ci.begin_suite("test")

        ### An alternative to deleting all suites before reloading
        ### is to simply replace the existing suite
        print("[1] Suspend the 'test' suite")
        ci.suspend("/test")

        print("[2] Replace the 'test' suite")
        ci.replace( "/test", "test.def")

    except RuntimeError as e:
        print("Failed:", e)



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from ecflow import Suite, Task, RepeatDate

if __name__ == "__main__":

    class ExperimentalSuite(object):
        def __init__(self, start, end):
            self.start_ = start
            self.end_ = end
            self.start_cycle_ = 12
            self.end_cycle_ = 12

        def generate(self):
            x_suite = Suite("x")
            make_fam = x_suite.add_family("make")
            make_fam.add_task("build")
            make_fam.add_task("more_work")

            main_fam = x_suite.add_family("main")
            main_fam.add_repeat(RepeatDate("YMD", self.start_, self.end_))
            main_fam.add_trigger("make == complete")

            previous = 0
            for FAM in (0, 6, 12, 18):
                fam_fam = x_suite.add_family(str(FAM))
                if FAM > 0:
                    fam_fam.add_trigger("./" + str(previous) + " == complete ")

                self.add_complete(fam_fam, FAM)

                fam_fam.add_task("run")
                fam_fam.add_task("run_more").add_trigger("run == complete")
                previous = FAM
            return x_suite

        def add_complete(self, family, fam):
            if fam < self.start_cycle_ and fam > self.end_cycle_:
                family.add_complete("../main:YMD eq " + str(self.start_) + " or ../main:YMD ge " + str(self.end_))
            elif fam < self.start_cycle_:
                family.add_complete("../main:YMD eq " + str(self.start_))
            elif fam > self.end_cycle_:
                family.add_complete("../main:YMD ge " + str(self.end_))
            return


    print(str(ExperimentalSuite(20050601, 20050605).generate()))

    # ==========================================================================

    # Control structure and looping
    var = "aa"
    if var in ("a", "aa", "aaa"):
        print("it is a kind of a ")
    elif var in ("b", "bb", "bb"):
        print("it is a kind of b ")
    else:
        print("it is something else ")


    # ====================================================================================

    # Reuseable class for adding synoptic times
    class VarAdder(object):
        def __init__(self, node):
            self.node = node

        def add(self, time):
            {
                6: lambda self: self.add6(time),
                12: lambda self: self.add12(time),
                18: lambda self: self.add18(time),
                24: lambda self: self.add24(time)
            }.get(time, self.errorHandler)(self)

        def add6(self, time):
            print("add6 " + str(time))
            self.node.add_today(17, 30)
            self.node.add_variable("ANTIME", str(time))

        def add12(self, time):
            print("add12 " + str(time))
            self.node.add_today(19, 15)
            self.node.add_variable("ANTIME", str(time))

        def add18(self, time):
            print("add18 " + str(time))
            self.node.add_time(1, 30)
            self.node.add_variable("ANTIME", str(time))

        def add24(self, time):
            print("add24 " + str(time))
            self.node.add_time(3, 0)
            self.node.add_variable("ANTIME", "0")
            self.node.add_variable("DELTA_DAY", "1")
            self.node.add_variable("EXPVER", "0002")

        def errorHandler(self, ignore): print("invalid time " + str(ignore))

for i in (0, 6):
    task = Task("t" + str(i))
    print(task.name())
    varAdder = VarAdder(task)
    varAdder.add(114)
    for var in task.variables:
        print(str(var) + "\n")
    for the_time in task.times:
        print(str(the_time) + "\n")
    for today in task.todays:
        print(str(today) + "\n")

# ==========================================================================

suite = Suite("x")
previous_time = 0
for i in (0, 6, 12, 18, 24):
    the_fam = suite.add_family(str(i))
    if i != 0:
        the_fam.add_trigger("./" + previous_time + " == complete ")
    the_fam.add_task("t1")
    the_fam.add_task("t2").add_trigger("t1 == complete")
    previous_time = str(i)



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import os
from ecflow import Defs, Suite, Variable, Limit, InLimit, Task, PartExpression, \
    Event, Meter, Label, RepeatInteger, RepeatEnumerated, RepeatDate, RepeatDateList, RepeatString, \
    TimeSlot, TimeSeries, Today, Time, Date, Day, Days, Cron, Autocancel, Late, \
    DState, Clock, ChildCmdType, ZombieType, ZombieAttr, ZombieUserActionType, Client, debug_build
import ecflow_test_util as Test

if __name__ == "__main__":

    Test.print_test_start(os.path.basename(__file__))

    #
    # Test for: See  ECFLOW-106 Times/Dates attributes attached to suite node
    #
    defs = Defs()
    suite = defs.add_suite("s1")

    #
    # Suite should not be allowed time based dependencies
    # Check Today
    expected_error = False
    try:
        suite.add_today("00:30")
    except RuntimeError:
        expected_error = True
    assert expected_error, "Suite should not allow any time based dependencies"

    expected_error = False
    try:
        suite.add_today(0, 30)
    except RuntimeError:
        expected_error = True
    assert expected_error, "Suite should not allow any time based dependencies"

    #
    # Check Time
    expected_error = False
    try:
        suite.add_time("+00:30")
    except RuntimeError:
        expected_error = True
    assert expected_error, "Suite should not allow any time based dependencies"

    expected_error = False
    try:
        suite.add_time(0, 30)
    except RuntimeError:
        expected_error = True
    assert expected_error, "Suite should not allow any time based dependencies"

    # 
    # Check Date::See  ECFLOW-106 Times/Dates attributes attached to suite node
    expected_error = False
    try:
        suite.add_date(1, 1, 2010)
    except RuntimeError:
        expected_error = True
    assert expected_error, "Suite should not allow any time based dependencies"

    # 
    # Check Day:: See  ECFLOW-106 Times/Dates attributes attached to suite node
    expected_error = False
    try:
        suite.add_day("sunday")
    except RuntimeError:
        expected_error = True
    assert expected_error, "Suite should not allow any time based dependencies"

    # 
    # Adding a RepeatDateList with a empty list should be an error
    expected_error = False
    task = Task("t")
    try:
        task.add_repeat(RepeatDateList("date", []))
    except RuntimeError:
        expected_error = True
    assert expected_error, "RepeatDateList adding an empty date list should be an error"

    print("All Tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from ecflow import Suite, Family, Task, Defs, Client, debug_build
import ecflow_test_util as Test
import os


def absNodePath(node):
    """ Given a node return its absolute node path. """
    """ Example of using get_parent()               """
    name_list = []
    name_list.append(node.name())
    parent = node.get_parent();
    while parent:
        name_list.append(parent.name())
        parent = parent.get_parent()

    name_list.reverse()
    ret = ""
    for name in name_list:
        ret = ret + "/"
        ret = ret + name
    return ret


if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    suite = Suite("s1");
    family = Family("f1")
    task = Task("t1")

    assert not suite.get_parent(), "Suite parent is always NULL"
    assert not family.get_parent(), "Expect no parent"
    assert not task.get_parent(), "Expect no parent"

    print("suite.get_defs() = " + str(suite.get_defs()))
    print("family.get_defs() = " + str(family.get_defs()))
    print("task.get_defs() = " + str(task.get_defs()))
    assert not suite.get_defs(), "Expected no defs, since suite not added to defs yet"
    assert not family.get_defs(), "Expected no defs"
    assert not task.get_defs(), "Expected no defs"

    suite.add_family(family)
    family.add_task(task)

    family2 = Family("f2")
    family.add_family(family2)

    t1 = Task("t1")
    family2.add_task(t1)

    defs = Defs()
    defs.add_suite(suite);

    print(absNodePath(t1))
    print(absNodePath(family2))
    print(absNodePath(family))
    print(absNodePath(suite))

    assert t1.get_abs_node_path() == absNodePath(t1), "Expected " + t1.get_abs_node_path() + " but got " + absNodePath(t1)
    assert family2.get_abs_node_path() == absNodePath(family2), "Expected " + family2.get_abs_node_path() + " but got " + absNodePath(family2)
    assert family.get_abs_node_path() == absNodePath(family), "Expected " + family.get_abs_node_path() + " but got " + absNodePath(family)
    assert suite.get_abs_node_path() == absNodePath(suite), "Expected " + family.get_abs_node_path() + " but got " + absNodePath(suite)
    print("All Tests pass")



#!/usr/bin/env python3

from ecflow import Client

if __name__ == "__main__":

    try:
        # The default is to contact localhost:3141, but will
        # use ECF_HOST and/or ECF_PORT env variables if set.
        ci = Client()
        ci.ping()

        # Explicitly set host and port (n.b. reuses the existing Client)
        ci.set_host_port("machineX:4141")
        ci.ping()

        # Explicitly set host and port when creating the Client
        ci = Client("oetzi:3444")
        ci.ping()

        # Ping inlined
        Client("polonius:4266").ping()

    except RuntimeError as e:
        print(f"ping failed: {e}", )



#!/usr/bin/env python3

import pathlib
from ecflow import Client

if __name__ == '__main__':

    base = pathlib.Path.home() / "course"

    try:
        ci = Client()

        print("[1] Suspend the 'test' suite")
        ci.suspend("/test")

        print("[2] Replace the 'test' suite")
        ci.replace( "/test", str(base / "test.def"))

    except RuntimeError as e:
        print("Failed:", e)



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
    Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
    JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
    Repeat, RepeatDate, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
    Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
    VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import ecflow_test_util as Test
import unittest
import shutil  # used to remove directory tree
import os, sys


def test_def_file():
    # return "test.def"
    return "test_tutorial_def_" + str(os.getpid()) + ".def"


def test_compile(text):
    # replace test.def in the text with test_def_file() so that we have a unique file per process
    text = text.replace('test.def', test_def_file())

    test_file = "py_u_test_tutorial_" + str(os.getpid()) + ".def"
    file = open(test_file, 'w')
    file.write(text)
    file.close()

    # execfile(test_file) only work for python 2.7
    with open(test_file) as f:
        code = compile(f.read(), test_file, 'exec')
        exec(code)

    os.remove(test_file)


def do_tear_down():
    Ecf.set_debug_equality(False)
    try:
        os.remove(test_def_file())
    except:
        pass


######################################################################################################
class TestNewSuite(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        home = os.path.join(os.getenv("HOME"), "course")
        self.defs = Defs()
        suite = self.defs.add_suite("test")
        suite.add_variable("ECF_HOME", home)
        suite.add_task("t1")

        with Defs() as self.defs2:
            with self.defs2.add_suite("test") as suite:
                suite.add_variable("ECF_HOME", home)
                suite.add_task("t1")

        with Defs() as self.defs3:
            self.defs3.add(
                Suite("test",
                      Edit(ECF_HOME=home),
                      Task("t1")))

        self.defs4 = Defs() + (Suite("test") + Edit(ECF_HOME=home))
        self.defs4.test += Task("t1")

        self.defs5 = Defs().add(
            Suite("test").add(
                Edit(ECF_HOME=home),
                Task("t1")))

        # print("Creating suite definition")
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs(
            Suite('test',
                  Task('t1'),
                  ECF_HOME=home))
        self.assertEqual(self.defs, defs, "defs not equal\n" + str(self.defs) + "\n" + str(defs))

        text = """import os
from ecflow import Defs,Suite,Task,Edit
   
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"),  "course")
defs = Defs( 
        Suite('test',
            Edit(ECF_HOME=home),
            Task('t1')))
#xx print(defs)

defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, defs, "defs not equal\n" + str(test_defs) + "\n" + str(defs))

    def test_defs_equal(self):
        self.assertEqual(self.defs, self.defs2, "defs not the equal")
        self.assertEqual(self.defs, self.defs3, "defs not the equal")
        self.assertEqual(self.defs, self.defs4, "defs not the equal")
        self.assertEqual(self.defs, self.defs5, "defs not the equal")

    def tearDown(self):
        do_tear_down()


######################################################################################################
class TestFamilies(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(
            Suite("test").add(
                Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home),
                Family("f1").add(
                    Task("t1"),
                    Task("t2"))))
        defs.save_as_defs(test_def_file())

        self.defs = defs

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit
         
def create_family_f1():
    return Family("f1",
                Task("t1"),
                Task("t2"))
            
#xx print("Creating suite definition") 
home = os.path.join(os.getenv("HOME"),  "course") 
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        # print("Creating suite definition")
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += Family("f1") + [Task("t{0}".format(i)) for i in range(1, 3)]
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestVariables(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f1():
            return Family("f1").add(
                Task("t1").add(Edit(SLEEP=20)),
                Task("t2").add(Edit(SLEEP=20)))

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
            Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home),
            create_family_f1()))

        defs.save_as_defs(test_def_file())

        self.defs = defs

    def test_me0(self):
        text = """#import os
from ecflow import Defs,Suite,Family,Task,Edit

def create_family_f1():
    return Family("f1",
                Task("t1",Edit(SLEEP=20)),
                Task("t2",Edit(SLEEP=20)))
  
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
        Suite("test", 
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))         
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def') 
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me2(self):
        import os
        home = os.path.join(os.getenv("HOME"), "course")

        with Defs() as defs:
            defs += Suite("test").add(
                Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home))
            defs.test += Family("f1").add(
                Task("t1").add(Edit(SLEEP=20)),
                Task("t2").add(Edit(SLEEP=20)))
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def test_me3(self):
        import os
        home = os.path.join(os.getenv("HOME"), "course")

        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += Family("f1") + (Task("t1") + Edit(SLEEP=20)) + (Task("t2") + Edit(SLEEP=20))
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestVariableInheritance(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f1():
            return Family("f1").add(
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2"))

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
            Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home),
            create_family_f1()))
        defs.save_as_defs(test_def_file())

        self.defs = defs;

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit

def create_family_f1():
    return Family("f1",
            Edit(SLEEP=20),
                Task("t1"),
                Task("t2"))
             
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        def create_family_f1():
            return Family("f1") + Edit(SLEEP=20) + Task("t1") + Task("t2")

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + (Suite("test") + create_family_f1())
        defs.test += [Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home)]
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestTriggers(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f1():
            return Family("f1").add(
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2").add(Trigger("t1 == complete")))

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
            Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home),
            create_family_f1()))
        defs.save_as_defs(test_def_file())

        self.defs = defs;

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger

def create_family_f1():
    return Family("f1",
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2",Trigger("t1 == complete")))
             
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        home = os.path.join(os.getenv("HOME"), "course")
        with Suite("test") as suite:
            suite += [Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home)]
            suite += Family("f1") + Edit(SLEEP=20) + Task("t1") + Task("t2")
            suite.f1.t2 += Trigger(["t1"])

        defs = Defs().add(suite)
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestEvents(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f1():
            return Family("f1").add(
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2").add(
                    Trigger("t1 == complete"),
                    Event("a"),
                    Event("b")),
                Task("t3").add(Trigger("t2:a")),
                Task("t4").add(Trigger("t2:b")))

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(
            Suite("test").add(
                Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home),
                create_family_f1()))
        defs.save_as_defs(test_def_file())

        self.defs = defs;

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Event

def create_family_f1():
    return Family("f1",
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2",
                    Trigger("t1 == complete"),
                    Event("a"),
                    Event("b")),
                Task("t3",
                    Trigger("t2:a")),
                Task("t4",
                    Trigger("t2:b")))
             
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        def create_family_f1():
            f1 = Family("f1") + [Edit(SLEEP=20),
                                 Task("t1"),
                                 Task("t2") + Trigger(["t1"]) + Event("a") + Event("b"),
                                 Task("t3") + Trigger("t2:a"),
                                 Task("t4") + Trigger("t2:b")]
            return f1

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + Suite("test")
        defs.test += [Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home), create_family_f1()]
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestComplete(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f1():
            return Family("f1").add(
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2").add(Trigger("t1 == complete"),
                               Event("a"), Event("b")),
                Task("t3").add(Trigger("t2:a")),
                Task("t4").add(Trigger("t2 == complete"),
                               Complete("t2:b")))

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
            Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home),
            create_family_f1()))
        defs.save_as_defs(test_def_file())

        self.defs = defs;

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event

def create_family_f1():
    return Family("f1",
                Edit(SLEEP= 20),
                Task("t1"),
                Task("t2",
                    Trigger("t1 == complete"),
                    Event("a"), 
                    Event("b")),
                Task("t3",
                    Trigger("t2:a")),
                Task("t4",
                    Trigger("t2 == complete"), 
                    Complete("t2:b")))
        
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        def create_family_f1():
            f1 = Family("f1") + Edit(SLEEP=20)
            f1 += [Task("t{0}".format(i)) for i in range(1, 5)]
            f1.t2 += [Trigger(["t1"]), Event("a"), Event("b")]
            f1.t3 += Trigger("t2:a")
            f1.t4 += [Trigger(["t2"]), Complete("t2:b")]
            return f1

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += create_family_f1()
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestMeter(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f1():
            return Family("f1").add(
                Edit(SLEEP=20),
                Task("t1").add(Meter("progress", 1, 100, 90)),
                Task("t2").add(Trigger("t1 == complete"),
                               Event("a"),
                               Event("b")),
                Task("t3").add(Trigger("t2:a")),
                Task("t4").add(Trigger("t2 == complete"),
                               Complete("t2:b")),
                Task("t5").add(Trigger("t1:progress ge 30")),
                Task("t6").add(Trigger("t1:progress ge 60")),
                Task("t7").add(Trigger("t1:progress ge 90")))

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
            Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home),
            create_family_f1()))
        defs.save_as_defs(test_def_file())

        self.defs = defs;

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter

def create_family_f1():
    return Family("f1",
                Edit(SLEEP= 20),
                Task("t1", Meter("progress", 1, 100, 90)),
                Task("t2", Trigger("t1 == complete"), Event("a"), Event("b")),
                Task("t3", Trigger("t2:a")),
                Task("t4", Trigger("t2 == complete"), Complete("t2:b")),
                Task("t5", Trigger("t1:progress ge 30")),
                Task("t6", Trigger("t1:progress ge 60")),
                Task("t7", Trigger("t1:progress ge 90")))  
        
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        def create_family_f1():
            f1 = Family("f1") + Edit(SLEEP=20)
            f1 += [Task("t{0}".format(i)) for i in range(1, 8)]
            f1.t1 += Meter("progress", 1, 100, 90)
            f1.t2 += [Trigger(["t1"]), Event("a"), Event("b")]
            f1.t3 += Trigger("t2:a")
            f1.t4 += [Trigger(["t2"]), Complete("t2:b")]
            f1.t5 += Trigger("t1:progress ge 30")
            f1.t6 += Trigger("t1:progress ge 60")
            f1.t7 += Trigger("t1:progress ge 90")
            return f1

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += create_family_f1()
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestTime(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f2():
            f2 = Family("f2")
            f2.add_variable("SLEEP", 20)
            f2.add_task("t1").add_time("00:30 23:30 00:30")  # start(hh:mm) end(hh:mm) increment(hh:mm)
            f2.add_task("t2").add_day("sunday")

            # for add_date(): day,month,year; here 0 means every month, and every year
            t3 = f2.add_task("t3")
            t3.add_date(1, 0, 0)  # day month year, first of every month or every year
            t3.add_time(12, 0)  # hour, minutes at 12 o'clock

            f2.add_task("t4").add_time(0, 2, True)  # hour, minutes, relative to suite start
            # 2 minutes after family f2 start
            f2.add_task("t5").add_time(0, 2)  # hour, minutes suite site
            # 2 minutes past midnight
            return f2

        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME", os.path.join(os.getenv("HOME"), "course"))
        suite.add_variable("ECF_INCLUDE", os.path.join(os.getenv("HOME"), "course"))

        # suite.add_family( create_family_f1() )
        suite.add_family(create_family_f2())
        errors = defs.check()
        assert len(errors) == 0, errors

        defs.save_as_defs(test_def_file())

        self.defs = defs;

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date

def create_family_f2():
    return Family("f2",
            Edit(SLEEP=20),
            Task("t1", Time("00:30 23:30 00:30")),     # start(hh:mm) end(hh:mm) increment(hh:mm)
            Task("t2", Day("sunday")),
            Task("t3", Date("1.*.*"), Time("12:00")),  # Date(day,month,year) - * means every day,month,year
            Task("t4", Time("+00:02")),                # + means realative to suite begin/requeue time
            Task("t5", Time("00:02")))                 # 2 minutes past midnight
 
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            #create_family_f1(),
            create_family_f2()
            ))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        def create_family_f2():
            f1 = Family("f2") + Edit(SLEEP=20)
            f1 += [Task("t{0}".format(i)) for i in range(1, 6)]
            f1.t1 += Time("00:30 23:30 00:30")  # start(hh:mm) end(hh:mm) increment(hh:mm)
            f1.t2 += Day("sunday")
            f1.t3 += [Date("1.*.*"),
                      Time("12:00")]
            f1.t4 += Time("+00:02")
            f1.t5 += Time("00:02")
            return f1

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += create_family_f2()
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def test_add(self):
        import os

        def create_family_f2():
            return Family("f2").add(
                Edit(SLEEP=20),
                Task("t1").add(Time("00:30 23:30 00:30")),
                Task("t2").add(Day("sunday")),
                Task("t3").add(Date("1.*.*"),
                               Time("12:00")),
                Task("t4").add(Time("+00:02")),
                Task("t5").add(Time("00:02")))

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
            Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home),
            create_family_f2()))
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestCron(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_house_keeping():
            cron = Cron()
            cron.set_week_days([0])
            cron.set_time_series("22:30")
            f2 = Family("house_keeping")
            f2.add_task("clear_log").add_cron(cron)
            return f2

        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME", os.path.join(os.getenv("HOME"), "course"))
        suite.add_variable("ECF_INCLUDE", os.path.join(os.getenv("HOME"), "course"))

        # suite.add_family( create_family_f1() )
        suite.add_family(create_family_house_keeping())
        errors = defs.check()
        assert len(errors) == 0, errors

        defs.save_as_defs(test_def_file())

        self.defs = defs;

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Cron
  
def create_family_house_keeping():
    return Family("house_keeping",
                Task("clear_log",
                    Cron("22:30",days_of_week=[0])))
          
print("Creating suite definition") 
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_house_keeping()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def tearDown(self):
        do_tear_down()


class TestIndentation(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        # version = sys.version_info
        # if version[1] < 7:
        #    #print "This example requires python version 2.7, but found : " + str(version)
        #    exit(0)

        with Defs() as defs:
            with defs.add_suite("test") as suite:
                suite += Edit(ECF_HOME=os.path.join(os.getenv("HOME"), "course"))
                suite += Edit(ECF_INCLUDE=os.path.join(os.getenv("HOME"), "course"))

            with suite.add_family("f1") as f1:
                f1 += Edit(SLEEP=20)
                f1 += Task("t1", Meter("progress", 1, 100, 90))
                f1 += Task("t2", Trigger("t1 == complete"), Event("a"), Event("b"))
                f1 += Task("t3", Trigger("t2:a"))
                f1 += Task("t4", Trigger("t2 == complete"), Complete("t2:b"))
                f1 += Task("t5", Trigger("t1:progress ge 30"))
                f1 += Task("t6", Trigger("t1:progress ge 60"))
                f1 += Task("t7", Trigger("t1:progress ge 90"))

            with suite.add_family("f2") as f2:
                f2 += Edit(SLEEP=20)
                f2 += Task("t1", Time("00:30 23:30 00:30"))
                f2 += Task("t2", Day("sunday"))
                f2 += Task("t3", Date(1, 0, 0), Time(12, 0))
                f2 += Task("t4", Time(0, 2, True))
                f2 += Task("t5", Time(0, 2))

        defs.save_as_defs(test_def_file())
        self.defs = defs;

    def test_preferred(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date

#xx print("Creating suite definition") 
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            Family("f1",
                Edit(SLEEP=20),
                Task("t1", Meter("progress", 1, 100, 90)),
                Task("t2", Trigger("t1 == complete"),Event("a"),Event("b")),
                Task("t3", Trigger("t2:a")),
                Task("t4", Trigger("t2 == complete"), Complete("t2:b")),
                Task("t5", Trigger("t1:progress ge 30")),
                Task("t6", Trigger("t1:progress ge 60")),
                Task("t7", Trigger("t1:progress ge 90"))),
            Family("f2",
                Edit(SLEEP=20),
                Task("t1", Time( "00:30 23:30 00:30" )),
                Task("t2", Day( "sunday" )),
                Task("t3", Date("1.*.*"), Time("12:00")),
                Task("t4", Time("+00:02")),
                Task("t5", Time("00:02")))))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        home = os.path.join(os.getenv("HOME"), "course")
        with Defs() as defs:
            with defs.add_suite("test") as suite:
                suite += [Edit(ECF_HOME=home), Edit(ECF_INCLUDE=home)]
                with suite.add_family("f1") as f1:
                    f1 += [Task("t{0}".format(i)) for i in range(1, 8)]
                    f1 += Edit(SLEEP=20)
                    f1.t1 += Meter("progress", 1, 100, 90)
                    f1.t2 += [Trigger(["t1"]), Event("a"), Event("b")]
                    f1.t3 += Trigger("t2:a")
                    f1.t4 += [Trigger(["t2"]), Complete("t2:b")]
                    f1.t5 += Trigger("t1:progress ge 30")
                    f1.t6 += Trigger("t1:progress ge 60")
                    f1.t7 += Trigger("t1:progress ge 90")
                with suite.add_family("f2") as f2:
                    f2 += [Edit(SLEEP=20), [Task("t{0}".format(i)) for i in range(1, 6)]]
                    f2.t1 += Time("00:30 23:30 00:30")
                    f2.t2 += Day("sunday")
                    f2.t3 += [Date("1.*.*"), Time("12:00")]
                    f2.t4 += Time("+00:02")
                    f2.t5 += Time("00:02")

        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs, defs, "defs not equal:\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        do_tear_down()


class TestLabel(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f3():
            fam = Family('f3')
            fam.add_task('t1').add_label("info", "")
            return fam

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME", home)
        suite.add_variable("ECF_INCLUDE", home)

        suite.add_family(create_family_f3())
        errors = defs.check()
        assert len(errors) == 0, errors

        defs.save_as_defs(test_def_file())

        self.defs = defs;

    def test_me(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label

def create_family_f3():
    return Family("f3",
                Task("t1",
                Label("info","")))
 
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f3()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def tearDown(self):
        do_tear_down()


class TestRepeat(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        self.ecf_home = File.build_dir() + "/libs/pyext/test/data/course_py" + str(sys.version_info[0]) + "_" + str(os.getpid())  # allow paralled runs
        self.ecf_includes = File.source_dir() + "/libs/pyext/test/data/includes"
        # print("self.ecf_home ",self.ecf_home )

        try:
            os.makedirs(self.ecf_home + "/test/f4/f5")
        except:
            pass

        t1_ecf = '%include <head.h>\n'
        t1_ecf += 'ecflow_client --label=info "My name is %NAME% My value is %VALUE% My date is %DATE%\n'
        t1_ecf += 'ecflow_client --label=date "year:%DATE_YYYY% month:%DATE_MM% day of month:%DATE_DD% day of week:%DATE_DOW%"\n'
        t1_ecf += 'sleep %SLEEP%\n'
        t1_ecf += '%include <tail.h>\n'

        self.t1_ecf_path = self.ecf_home + "/test/f4/f5/t1.ecf"
        # print("self.t1_ecf_path",self.t1_ecf_path)
        file = open(self.t1_ecf_path, "w")
        file.write(t1_ecf)
        file.close()

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        do_tear_down()
        os.remove(self.t1_ecf_path)
        shutil.rmtree(self.ecf_home, ignore_errors=True)

    def test_repeat0(self):

        def create_family_f4():
            return Family("f4",
                          Edit(SLEEP=2),
                          RepeatString("NAME", ["a", "b", "c", "d", "e", "f"]),
                          Family("f5",
                                 RepeatInteger("VALUE", 1, 10),
                                 Task("t1",
                                      RepeatDate("DATE", 20101230, 20110105),
                                      Label("info", ""),
                                      Label("date", ""))))

        defs = Defs(
            Suite("test",
                  Edit(ECF_HOME=self.ecf_home, ECF_INCLUDE=self.ecf_includes),
                  create_family_f4()))
        # print(defs)

        result = defs.check_job_creation()
        self.assertEqual(result, "", "expected job creation to succeed " + result)

        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label, \
                   RepeatString,RepeatInteger,RepeatDate
        
def create_family_f4():
    return Family("f4",
                Edit(SLEEP=2), 
                RepeatString("NAME", ["a", "b", "c", "d", "e", "f" ]),
                Family("f5",
                    RepeatInteger("VALUE", 1, 10),
                    Task("t1",
                        RepeatDate("DATE", 20101230, 20110105),
                        Label("info",""),
                        Label("date",""))))
 
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f4()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0,defs.check() 

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)

    def test_repeat(self):

        # print("Creating suite definition")
        defs = Defs(
            Suite("test",
                  Edit(ECF_HOME=self.ecf_home, ECF_INCLUDE=self.ecf_includes),
                  Family("f4",
                         Edit(SLEEP=2),
                         RepeatString("NAME", ["a", "b", "c", "d", "e", "f"]),
                         Family("f5",
                                RepeatInteger("VALUE", 1, 10),
                                Task("t1",
                                     RepeatDate("DATE", 20101230, 20110105),
                                     Label("info", ""),
                                     Label("date", ""))))))
        # print(defs)

        result = defs.check_job_creation()
        self.assertEqual(result, "", "expected job creation to succeed " + result)

        defs.save_as_defs(test_def_file())

    def test_repeat3(self):

        defs = Defs().add(Suite("test"))
        defs.test += [{"ECF_INCLUDE": self.ecf_includes, "ECF_HOME": self.ecf_home},
                      Family("f4")]
        defs.test.f4 += [Edit(SLEEP=2),
                         RepeatString("NAME", ["a", "b", "c", "d", "e", "f"]),
                         Family("f5")]
        defs.test.f4.f5 += [RepeatInteger("VALUE", 1, 10),
                            Task("t1")]
        defs.test.f4.f5.t1 += [RepeatDate("DATE", 20101230, 20110105),
                               Label("info", ""),
                               Label("date", "")]
        # print(defs)
        result = defs.check_job_creation()
        self.assertEqual(result, "", "expected job creation to succeed " + result)

        defs.save_as_defs(test_def_file())


class TestLimit(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f5():
            f5 = Family("f5")
            f5.add_inlimit("l1")
            f5.add_variable("SLEEP", 20)
            for i in range(1, 10):
                f5.add_task("t" + str(i))
            return f5

        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME", os.path.join(os.getenv("HOME"), "course"))
        suite.add_variable("ECF_INCLUDE", os.path.join(os.getenv("HOME"), "course"))

        suite.add_limit("l1", 2)
        suite.add_family(create_family_f5())

        defs.save_as_defs(test_def_file())

        self.defs = defs

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label, \
                   RepeatString,RepeatInteger,RepeatDate,InLimit,Limit
         
def create_family_f5() :
    return Family("f5",
            InLimit("l1"),
            Edit(SLEEP=20),
            [ Task('t{0}'.format(i)) for i in range(1,10) ] )
     
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"),"course")
defs = Defs(
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            Limit("l1",2),
            create_family_f5()))
#xx print(defs)
 
#xx print("Checking job creation: .ecf -> .job0") 
#print(defs.check_job_creation())
 
#xx print("Checking trigger expressions")
assert len(defs.check()) == 0,defs.check()
 
#xx print("Saving definition to file 'test.def'")
defs.save_as_defs(test_def_file()) 
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def tearDown(self):
        do_tear_down()


class TestLateAttribute(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_family_f6():
            f6 = Family("f6")
            f6.add_variable("SLEEP", 120)
            t1 = f6.add_task("t1")
            late = Late()
            late.complete(0, 1, True)  # hour,minute,relative,    set late flag if task take longer than a minute
            t1.add_late(late)
            return f6

        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME", os.path.join(os.getenv("HOME"), "course"))
        suite.add_variable("ECF_INCLUDE", os.path.join(os.getenv("HOME"), "course"))
        suite.add_family(create_family_f6())
        # print(defs)

        assert len(defs.check()) == 0, defs.check()

        defs.save_as_defs(test_def_file())

        self.defs = defs

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label, \
                   RepeatString,RepeatInteger,RepeatDate,InLimit,Limit,Late
        
def create_family_f6():
    # set late flag if task t1 takes longer than a minute
    return Family("f6",
                Edit(SLEEP=120),
                Task("t1",
                    Late(complete='+00:01')))
     
#xx print("Creating suite definition")   
home = os.path.join(os.getenv("HOME"),"course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f6()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0,defs.check() 

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def tearDown(self):
        do_tear_down()


class TestPythonScripting(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_suite(name):
            suite = Suite(name)
            for i in range(1, 7):
                fam = suite.add_family("f" + str(i))
                for t in ("a", "b", "c", "d", "e"):
                    fam.add_task(t)
            return suite

        self.defs = Defs(create_suite('s1'))

    def test_me(self):
        def create_suite(name):
            return Suite(name,
                         [Family("f{0}".format(i),
                                 [Task(t) for t in ("a", "b", "c", "d", "e")])
                          for i in range(1, 7)])

        defs = Defs(create_suite('s1'))

        self.assertEqual(self.defs, defs, "defs not equal\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        Ecf.set_debug_equality(False)


class TestPythonScripting2(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        def create_sequential_suite(name):
            suite = Suite(name)
            for i in range(1, 7):
                fam = suite.add_family("f" + str(i))
                if i != 1:
                    fam.add_trigger("f" + str(i - 1) + " == complete")  # or fam.add_family( "f%d == complete" % (i-1) )
                for t in ("a", "b", "c", "d", "e"):
                    fam.add_task(t)
            return suite

        self.defs = Defs(create_sequential_suite('s1'))

    def test_me(self):
        def create_sequential_suite(name):
            suite = Suite(name)
            for i in range(1, 7):
                fam = suite.add_family("f" + str(i))
                if i != 1:
                    fam += Trigger("f" + str(i - 1) + " == complete")  # or fam.add_family( "f%d == complete" % (i-1) )
                for t in ("a", "b", "c", "d", "e"):
                    fam.add_task(t)
            return suite

        defs = Defs(create_sequential_suite('s1'))

        self.assertEqual(self.defs, defs, "defs not equal\n" + str(self.defs) + "\n" + str(defs))

    def tearDown(self):
        Ecf.set_debug_equality(False)


class TestDataAquistionSolution(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        defs = Defs()
        suite = defs.add_suite("data_aquisition")
        suite.add_repeat(RepeatDay(1))
        suite.add_variable("ECF_HOME", os.getenv("HOME") + "/course")
        suite.add_variable("ECF_INCLUDE", os.getenv("HOME") + "/course")
        suite.add_variable("ECF_FILES", os.getenv("HOME") + "/course/data")
        suite.add_variable("SLEEP", "2")
        for city in ("Exeter", "Toulouse", "Offenbach", "Washington", "Tokyo", "Melbourne", "Montreal"):
            fcity = suite.add_family(city)
            fcity.add_task("archive")
            for obs_type in ("observations", "fields", "images"):
                type_fam = fcity.add_family(obs_type)
                if city in ("Exeter", "Toulouse", "Offenbach"): type_fam.add_time("00:00 23:00 01:00")
                if city in ("Washington"):                     type_fam.add_time("00:00 23:00 03:00")
                if city in ("Tokyo"):                          type_fam.add_time("12:00")
                if city in ("Melbourne"):                      type_fam.add_day("monday")
                if city in ("Montreal"):                       type_fam.add_date(1, 0, 0)

                type_fam.add_task("get")
                type_fam.add_task("process").add_trigger("get eq complete")
                type_fam.add_task("store").add_trigger("get eq complete")

        # print(defs)
        self.defs = defs

    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label, \
                   RepeatString,RepeatInteger,RepeatDate

home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
        Suite("data_aquisition",
            RepeatDay(1),
            Edit(ECF_HOME=home),
            Edit(ECF_INCLUDE=home),
            Edit(ECF_FILES=home + "/data"),
            Edit(SLEEP=2)))
for city in ( "Exeter", "Toulouse", "Offenbach", "Washington", "Tokyo", "Melbourne", "Montreal" ) :
    fcity = defs.data_aquisition.add_family(city)
    fcity += Task("archive")
    for obs_type in ( "observations", "fields", "images" ):
        type_fam = fcity.add_family(obs_type)
        if city in ("Exeter", "Toulouse", "Offenbach"): type_fam + Time("00:00 23:00 01:00")
        if city in ("Washington") :                     type_fam + Time("00:00 23:00 03:00")
        if city in ("Tokyo") :                          type_fam + Time("12:00")
        if city in ("Melbourne") :                      type_fam + Day( "monday" )
        if city in ("Montreal") :                       type_fam + Date(1, 0, 0)
         
        type_fam + Task("get") + Task("process",Trigger("get eq complete")) + Task("store",Trigger("get eq complete"))         
 
#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs, self.defs, "defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + Suite("data_aquisition").add(
            RepeatDay(1),
            Edit(ECF_HOME=home),
            Edit(ECF_INCLUDE=home),
            Edit(ECF_FILES=home + "/data"),
            Edit(SLEEP=2))
        for city in ("Exeter", "Toulouse", "Offenbach", "Washington", "Tokyo", "Melbourne", "Montreal"):
            fcity = defs.data_aquisition.add_family(city)
            fcity += Task("archive")
            for obs_type in ("observations", "fields", "images"):
                type_fam = fcity.add_family(obs_type)
                if city in ("Exeter", "Toulouse", "Offenbach"): type_fam + Time("00:00 23:00 01:00")
                if city in ("Washington"):                     type_fam + Time("00:00 23:00 03:00")
                if city in ("Tokyo"):                          type_fam + Time("12:00")
                if city in ("Melbourne"):                      type_fam + Day("monday")
                if city in ("Montreal"):                       type_fam + Date(1, 0, 0)

                type_fam += [Task("get"), Task("process"), Task("store")]
                type_fam.process += Trigger("get eq complete")
                type_fam.store += Trigger("get eq complete")

        self.assertEqual(self.defs, defs, "defs not equal")

    def tearDown(self):
        Ecf.set_debug_equality(False)


class TestOperationalSolution(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        defs = Defs()
        suite = defs.add_suite("operation_suite")
        suite.add_repeat(RepeatDay(1))
        suite.add_variable("ECF_HOME", os.getenv("HOME") + "/course")
        suite.add_variable("ECF_INCLUDE", os.getenv("HOME") + "/course")
        suite.add_variable("ECF_FILES", os.getenv("HOME") + "/course/oper")

        # Defines the triggers for the first cycle
        cycle_triggers = "1"
        for cycle in ("00", "12"):

            if cycle == "12":
                last_step = 240
            else:
                last_step = 24

            fcycle_fam = suite.add_family(cycle)
            fcycle_fam.add_variable("CYCLE", cycle)
            fcycle_fam.add_variable("LAST_STEP", last_step)

            if cycle_triggers != "1":
                fcycle_fam.add_trigger(cycle_triggers)

            analysis_fam = fcycle_fam.add_family("analysis")
            analysis_fam.add_task("get_observations")
            analysis_fam.add_task("run_analysis").add_trigger("get_observations == complete")
            analysis_fam.add_task("post_processing").add_trigger("run_analysis == complete")

            forecast_fam = fcycle_fam.add_family("forecast")
            forecast_fam.add_trigger("analysis == complete")
            forecast_fam.add_task("get_input_data")
            run_forecast_task = forecast_fam.add_task("run_forecast")
            run_forecast_task.add_trigger("get_input_data == complete")
            run_forecast_task.add_meter("step", 0, last_step, last_step)

            archive_fam = fcycle_fam.add_family("archive")
            fam_analsis = archive_fam.add_family("analysis")
            fam_analsis.add_variable("TYPE", "analysis")
            fam_analsis.add_variable("STEP", "0")
            fam_analsis.add_trigger("../analysis/run_analysis == complete")
            fam_analsis.add_task("save")

            for i in range(6, last_step + 1, 6):
                step_fam = fam_analsis.add_family("step_" + str(i))
                step_fam.add_variable("TYPE", "forecast")
                step_fam.add_variable("STEP", i)
                step_fam.add_trigger("../../forecast/run_forecast:step ge " + str(i))
                step_fam.add_task("save")

            # Defines the triggers for the next cycle
            cycle_triggers = "./" + cycle + " == complete"

            # print(defs)
        self.defs = defs

    def test_sol1(self):
        import os

        home = os.getenv("HOME") + "/course"
        cycle_triggers = None
        last_step = {"12": 240,
                     "00": 24, }

        def cycle_trigger(cycle):
            if cycle == "12": return Trigger("./00 == complete")
            return None

        defs = Defs(
            Suite("operation_suite",
                  RepeatDay(1),
                  Edit(ECF_HOME=home),
                  Edit(ECF_INCLUDE=home),
                  Edit(ECF_FILES=home + "/oper"),
                  [Family(cycle,
                          Edit(CYCLE=cycle),
                          Edit(LAST_STEP=last_step[cycle]),

                          cycle_trigger(cycle),

                          Family("analysis",
                                 Task("get_observations"),
                                 Task("run_analysis", Trigger(["get_observations"])),
                                 Task("post_processing", Trigger(["run_analysis"]))
                                 ),

                          Family("forecast",
                                 Trigger(["analysis"]),
                                 Task("get_input_data"),
                                 Task("run_forecast",
                                      Trigger(["get_input_data"]),
                                      Meter("step", 0, last_step[cycle])),
                                 ),

                          Family("archive",
                                 Family("analysis",
                                        Edit(TYPE="analysis"),
                                        Edit(STEP=0),
                                        Trigger("../analysis/run_analysis == complete"),
                                        Task("save"),
                                        [Family("step_{0}".format(i),
                                                Edit(TYPE="forecast"),
                                                Edit(STEP=i),
                                                Trigger("../../forecast/run_forecast:step ge {0}".format(i)),
                                                Task("save"))
                                         for i in range(6, last_step[cycle] + 1, 6)]
                                        )
                                 )
                          ) for cycle in ("00", "12")]
                  )
        )
        # print(defs)

        self.assertEqual(self.defs, defs, "defs not equal")

    def tearDown(self):
        Ecf.set_debug_equality(False)


class TestBackArchivingSolution(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)

        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs()
        suite = defs.add_suite("back_archiving")
        suite.add_repeat(RepeatDay(1))
        suite.add_variable("ECF_HOME", home)
        suite.add_variable("ECF_INCLUDE", home)
        suite.add_variable("ECF_FILES", home + "/back")
        suite.add_variable("SLEEP", "2")
        suite.add_limit("access", 2)
        for kind in ("analysis", "forecast", "climatology", "observations", "images"):
            find_fam = suite.add_family(kind)
            find_fam.add_repeat(RepeatDate("DATE", 19900101, 19950712))
            find_fam.add_variable("KIND", kind)
            find_fam.add_task("get_old").add_inlimit("access")
            find_fam.add_task("convert").add_trigger("get_old == complete")
            find_fam.add_task("save_new").add_trigger("convert == complete")

        # print(defs)

        self.defs = defs

    def test_sol1(self):
        import os
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs(
            Suite("back_archiving",
                  RepeatDay(1),
                  Edit(ECF_HOME=home),
                  Edit(ECF_INCLUDE=home),
                  Edit(ECF_FILES=home + "/back"),
                  Edit(SLEEP=2),
                  Limit("access", 2),
                  [Family(kind,
                          RepeatDate("DATE", 19900101, 19950712),
                          Edit(KIND=kind),
                          Task("get_old", InLimit("access")),
                          Task("convert", Trigger("get_old == complete")),
                          Task("save_new", Trigger("convert == complete")))
                   for kind in ("analysis", "forecast", "climatology", "observations", "images")]))
        # print(defs)

        self.assertEqual(self.defs, defs, "defs not equal")

    def tearDown(self):
        Ecf.set_debug_equality(False)


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
    Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
    JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
    Repeat, RepeatDate, RepeatDateList, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
    Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
    VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest
import sys
import os


class Test_get_attr(unittest.TestCase):
    def test_get_attr(self):
        defs = Defs() + (Suite('s') + Family('f').add(Task('t') + Edit(var="1")))
        defs += Edit(var="1")

        self.assertIsInstance(defs.s, Suite, "Expected suite")
        self.assertIsInstance(defs.s.f, Family, "Expected family")
        self.assertIsInstance(defs.s.f.t, Task, "Expected Task but found " + str(type(defs.s.f.t)))

        self.assertIsInstance(defs.var, Variable, "Expected Variable but found " + str(type(defs.var)))
        self.assertIsInstance(defs.s.f.t.var, Variable, "Expected Variable but found " + str(type(defs.s.f.t.var)))

    def test_get_attr_generated_variables(self):
        defs = Defs() + (Suite('s') + Family('f').add((Task('t') + Edit(var="1") + RepeatDate("YMD", 20100111, 20100115, 2)),
                                                      (Task('t2') + Edit(var="1") + RepeatDateList("YMD", [20100111, 20100115]))))
        defs.s.f.t += Meter("meter", 0, 100)
        defs.s.f.t += Event("event")
        defs.s.f.t += Limit("limitx", 10)
        # PrintStyle.set_style(Style.STATE)
        # print(defs)

        self.assertTrue(defs.ECF_MICRO, "expected generated variable")
        self.assertTrue(defs.ECF_HOME, "expected generated variable")
        self.assertTrue(defs.ECF_JOB_CMD, "expected generated variable")
        self.assertTrue(defs.ECF_KILL_CMD, "expected generated variable")
        self.assertTrue(defs.ECF_STATUS_CMD, "expected generated variable")
        self.assertTrue(defs.ECF_URL_CMD, "expected generated variable")
        self.assertTrue(defs.ECF_LOG, "expected generated variable")
        self.assertTrue(defs.ECF_INTERVAL, "expected generated variable")
        self.assertTrue(defs.ECF_LISTS, "expected generated variable")
        self.assertTrue(defs.ECF_CHECK, "expected generated variable")
        self.assertTrue(defs.ECF_CHECKOLD, "expected generated variable")
        self.assertTrue(defs.ECF_CHECKINTERVAL, "expected generated variable")
        self.assertTrue(defs.ECF_CHECKMODE, "expected generated variable")
        self.assertTrue(defs.ECF_TRIES, "expected generated variable")
        self.assertTrue(defs.ECF_VERSION, "expected generated variable")
        self.assertTrue(defs.ECF_PORT, "expected generated variable")
        self.assertTrue(defs.ECF_HOST, "expected generated variable")

        self.assertTrue(defs.s.SUITE, "expected generated variable")
        self.assertEqual(defs.s.SUITE.value(), 's', "expected suite name of 's' but found")
        self.assertTrue(defs.s.ECF_DATE, "expected generated variable")
        self.assertTrue(defs.s.YYYY, "expected generated variable")
        self.assertTrue(defs.s.DOW, "expected generated variable")
        self.assertTrue(defs.s.DOY, "expected generated variable")
        self.assertTrue(defs.s.DATE, "expected generated variable")
        self.assertTrue(defs.s.DAY, "expected generated variable")
        self.assertTrue(defs.s.DD, "expected generated variable")
        self.assertTrue(defs.s.MM, "expected generated variable")
        self.assertTrue(defs.s.MONTH, "expected generated variable")
        self.assertTrue(defs.s.ECF_CLOCK, "expected generated variable")
        self.assertTrue(defs.s.ECF_TIME, "expected generated variable")
        self.assertTrue(defs.s.TIME, "expected generated variable")

        self.assertTrue(defs.s.f.FAMILY, "expected generated variable")
        self.assertTrue(defs.s.f.FAMILY1, "expected generated variable")

        self.assertTrue(defs.s.f.t.TASK, "expected generated variable")
        self.assertEqual(defs.s.f.t.TASK.value(), 't', "expected task name of 's'")
        self.assertTrue(defs.s.f.t.ECF_JOB, "expected generated variable")
        self.assertTrue(defs.s.f.t.ECF_SCRIPT, "expected generated variable")
        self.assertTrue(defs.s.f.t.ECF_JOBOUT, "expected generated variable")
        self.assertTrue(defs.s.f.t.ECF_TRYNO, "expected generated variable")
        self.assertEqual(defs.s.f.t.ECF_TRYNO.value(), '0', "expected task try no of '0'")
        self.assertTrue(defs.s.f.t.ECF_RID, "expected generated variable")
        self.assertTrue(defs.s.f.t.ECF_NAME, "expected generated variable")
        self.assertEqual(defs.s.f.t.ECF_NAME.value(), '/s/f/t', "expected task ECF_NAME of '/s/f/t'")
        self.assertTrue(defs.s.f.t.ECF_PASS, "expected generated variable")

        self.assertEqual(defs.s.f.t.YMD.value(), '20100111', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_YYYY.value(), '2010', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_MM.value(), '1', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_DD.value(), '11', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_DOW.value(), '1', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_JULIAN.value(), '2455208', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.event.value(), 0, "expected generated event of value 0 but found " + str(defs.s.f.t.event.value()))
        self.assertEqual(defs.s.f.t.meter.value(), 0, "expected generated meter of value 0 but found " + str(defs.s.f.t.meter.value()))
        self.assertEqual(defs.s.f.t.limitx.value(), 0, "expected generated limit of value 0 but found " + str(defs.s.f.t.limitx.value()))  #

        self.assertEqual(defs.s.f.t2.YMD.value(), '20100111', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_YYYY.value(), '2010', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_MM.value(), '1', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_DD.value(), '11', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_DOW.value(), '1', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_JULIAN.value(), '2455208', "expected generated YMD of value")


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#!/usr/bin/env python3

import pathlib
from ecflow import Defs

if __name__ == '__main__':

    base = pathlib.Path.home() / "course"

    print("[1] Load suite definition from file 'test.def'")
    defs = Defs(str(base / "test.def"))
    print(defs)

    print("[2] Validating job creation: .ecf -> .job0")
    defs.check_job_creation(throw_on_error=True, verbose=True)
